'use strict';

var deepmerge = require('deepmerge');
var coreNodejs = require('./core-nodejs.js');
var core = require('./core.js');
var jestDiffDefault = require('jest-diff');
var kleur = require('kleur');
var jestSnapshot = require('jest-snapshot');

function _interopDefault (e) { return e && e.__esModule ? e : { default: e }; }

function _interopNamespace(e) {
    if (e && e.__esModule) return e;
    var n = Object.create(null);
    if (e) {
        Object.keys(e).forEach(function (k) {
            if (k !== 'default') {
                var d = Object.getOwnPropertyDescriptor(e, k);
                Object.defineProperty(n, k, d.get ? d : {
                    enumerable: true,
                    get: function () { return e[k]; }
                });
            }
        });
    }
    n.default = e;
    return Object.freeze(n);
}

var deepmerge__default = /*#__PURE__*/_interopDefault(deepmerge);
var jestDiffDefault__namespace = /*#__PURE__*/_interopNamespace(jestDiffDefault);
var kleur__default = /*#__PURE__*/_interopDefault(kleur);

/* eslint-disable @typescript-eslint/ban-ts-comment -- this code needs to work
 * with multiple different versions of jest and it does verification of which
 * one is actually present but the other variants will cause errors, as is expected */
/* eslint-disable @typescript-eslint/prefer-ts-expect-error -- one of the
 * variants will always not be an error */
/* ignore typing for compatibility so it will seem "impossible" but different
 * version will yield different source */
/* istanbul ignore next: this is covered by integration tests */
const diffCandidates = [
    // @ts-ignore
    jestDiffDefault__namespace.default === null || jestDiffDefault__namespace.default === void 0 ? void 0 : jestDiffDefault__namespace.default.diff,
    // @ts-ignore
    jestDiffDefault__namespace.default,
    // @ts-ignore
    jestDiffDefault__namespace === null || jestDiffDefault__namespace === void 0 ? void 0 : jestDiffDefault__namespace.diff,
    // @ts-ignore
    jestDiffDefault__namespace,
];
const isFunction = (fn) => typeof fn === "function";
/* eslint-disable-next-line @typescript-eslint/no-non-null-assertion -- assume
 * one of the candidate matches, there will be a reasonable error later on if
 * not */
const diff = diffCandidates.find(isFunction);

function isThenable(src) {
    return src && typeof src === "object" && typeof src.then === "function";
}

/**
 * Creates a wrapped function based on the passed function.
 *
 * The returned function takes either a `T` or `Promise<T>`. If `T` the result
 * will be synchronous or if `Promise<T>` the result will be asynchronous.
 *
 * In practice this means that if you pass a synchronous object into it you will
 * maintain synchronous code but if you pass an asynchronous object you must
 * await the result.
 */
function diverge(fn) {
    function diverged(actual, ...args) {
        if (isThenable(actual)) {
            return actual.then((resolved) => fn.call(this, resolved, ...args));
        }
        else {
            return fn.call(this, actual, ...args);
        }
    }
    return diverged;
}

/**
 * Takes all messages from all files and flattens to a single array.
 */
function flattenMessages(report) {
    return report.results.reduce((aggregated, result) => {
        return aggregated.concat(result.messages);
    }, []);
}

function toBeValid(report) {
    if (report.valid) {
        return {
            pass: true,
            message: /* istanbul ignore next */ () => "Result should not contain error",
        };
    }
    else {
        const firstError = report.results[0].messages[0];
        return {
            pass: false,
            message: () => `Result should be valid but had error "${firstError.message}"`,
        };
    }
}
var toBeValid$1 = diverge(toBeValid);

function toBeInvalid(report) {
    if (report.valid) {
        return {
            pass: false,
            message: () => "Result should be invalid but had no errors",
        };
    }
    else {
        return {
            pass: true,
            message: /* istanbul ignore next */ () => "Result should not contain error",
        };
    }
}
var toBeInvalid$1 = diverge(toBeInvalid);

function isMessage(arg) {
    if (!arg) {
        return false;
    }
    return Boolean(arg.ruleId ||
        arg.severity ||
        arg.message ||
        arg.offset ||
        arg.line ||
        arg.column ||
        arg.size ||
        arg.selector ||
        arg.context);
}
function isConfig(arg) {
    if (!arg) {
        return false;
    }
    return Boolean(arg.root || arg.extends || arg.elements || arg.plugin || arg.transform || arg.rules);
}
function isString(arg) {
    return typeof arg === "string";
}
function getMarkup(src) {
    if (typeof HTMLElement !== "undefined" && src instanceof HTMLElement) {
        return src.outerHTML;
    }
    /* istanbul ignore else: prototype only allows string or HTMLElement */
    if (typeof src === "string") {
        return src;
    }
    else {
        throw new Error(`Failed to get markup from "${typeof src}" argument`);
    }
}
function toHTMLValidate(actual, arg0, arg1, arg2) {
    const markup = getMarkup(actual);
    const message = isMessage(arg0) ? arg0 : undefined;
    const config = isConfig(arg0) ? arg0 : isConfig(arg1) ? arg1 : undefined;
    const filename = isString(arg0) ? arg0 : isString(arg1) ? arg1 : arg2;
    return toHTMLValidateImpl.call(this, markup, message, config, filename);
}
function toHTMLValidateImpl(actual, expectedError, userConfig, filename) {
    var _a;
    const defaultConfig = {
        rules: {
            /* jsdom normalizes style so disabling rule when using this matcher or it
             * gets quite noisy when configured with self-closing */
            "void-style": "off",
        },
    };
    const config = deepmerge__default.default(defaultConfig, userConfig !== null && userConfig !== void 0 ? userConfig : {});
    /* istanbul ignore next: cant figure out when this would be unset */
    const actualFilename = (_a = filename !== null && filename !== void 0 ? filename : this.testPath) !== null && _a !== void 0 ? _a : "inline";
    const loader = new coreNodejs.FileSystemConfigLoader({
        extends: ["html-validate:recommended"],
    });
    const htmlvalidate = new core.HtmlValidate(loader);
    const report = htmlvalidate.validateStringSync(actual, actualFilename, config);
    const pass = report.valid;
    if (pass) {
        return { pass, message: () => "HTML is valid when an error was expected" };
    }
    else {
        if (expectedError) {
            /* eslint-disable-next-line @typescript-eslint/no-unsafe-assignment -- outside our control, this is what jest gives us back and it gladly accepts it back */
            const matcher = expect.arrayContaining([expect.objectContaining(expectedError)]);
            const errorPass = this.equals(report.results[0].messages, matcher);
            const diffString = diff(matcher, report.results[0].messages, {
                expand: this.expand,
                aAnnotation: "Expected error",
                bAnnotation: "Actual error",
            });
            const hint = this.utils.matcherHint(".not.toHTMLValidate", undefined, undefined, {
                comment: "expected error",
            });
            const expectedErrorMessage = () => [
                hint,
                "",
                "Expected error to be present:",
                this.utils.printExpected(expectedError),
                /* istanbul ignore next */ diffString ? `\n${diffString}` : "",
            ].join("\n");
            return { pass: !errorPass, message: expectedErrorMessage };
        }
        const errors = report.results[0].messages.map((message) => `  ${message.message} [${message.ruleId}]`);
        return {
            pass,
            message: () => ["Expected HTML to be valid but had the following errors:", ""].concat(errors).join("\n"),
        };
    }
}
var toHTMLValidate$1 = diverge(toHTMLValidate);

function toHaveErrorImpl(jest, actual, expected) {
    const flattened = flattenMessages(actual);
    const matcher = [expect.objectContaining(expected)];
    const pass = jest.equals(flattened, matcher);
    const diffString = diff(matcher, flattened, { expand: jest.expand });
    const hint = jest.utils.matcherHint(".toHaveError");
    const prettyExpected = jest.utils.printExpected(matcher);
    const prettyReceived = jest.utils.printReceived(flattened);
    const resultMessage = () => {
        return [
            hint,
            "",
            "Expected error to equal:",
            `  ${prettyExpected}`,
            "Received:",
            `  ${prettyReceived}`,
            /* istanbul ignore next */ diffString ? `\nDifference:\n\n${diffString}` : "",
        ].join("\n");
    };
    return { pass, message: resultMessage };
}
function toHaveError(actual, arg1, arg2, arg3) {
    if (typeof arg1 === "string") {
        const expected = {
            ruleId: arg1,
            message: arg2,
        };
        if (arg3) {
            /* eslint-disable-next-line @typescript-eslint/no-unsafe-assignment -- this is supposed to accept anything */
            expected.context = arg3;
        }
        return toHaveErrorImpl(this, actual, expected);
    }
    else {
        return toHaveErrorImpl(this, actual, arg1);
    }
}
var toHaveError$1 = diverge(toHaveError);

/* eslint-disable prefer-template -- technical debt, should be refactored */
function toHaveErrors(report, errors) {
    const flattened = flattenMessages(report);
    const matcher = errors.map((entry) => {
        if (Array.isArray(entry)) {
            const [ruleId, message] = entry;
            /* eslint-disable-next-line @typescript-eslint/no-unsafe-return -- jest type is declared like this */
            return expect.objectContaining({ ruleId, message });
        }
        else {
            /* eslint-disable-next-line @typescript-eslint/no-unsafe-return -- jest type is declared like this  */
            return expect.objectContaining(entry);
        }
    });
    const pass = this.equals(flattened, matcher);
    const diffString = diff(matcher, flattened, { expand: this.expand });
    const resultMessage = () => this.utils.matcherHint(".toHaveErrors") +
        "\n\n" +
        "Expected error to equal:\n" +
        `  ${this.utils.printExpected(matcher)}\n` +
        "Received:\n" +
        `  ${this.utils.printReceived(flattened)}` +
        /* istanbul ignore next */ (diffString ? `\n\nDifference:\n\n${diffString}` : "");
    return { pass, message: resultMessage };
}
var toHaveErrors$1 = diverge(toHaveErrors);

/**
 * @internal
 */
function getResults(filename, value) {
    if (typeof value === "string") {
        const loader = new coreNodejs.FileSystemConfigLoader({
            extends: ["html-validate:recommended"],
        });
        const htmlvalidate = new core.HtmlValidate(loader);
        const report = htmlvalidate.validateStringSync(value, filename, {
            rules: {
                "void-style": "off",
            },
        });
        return report.results.map((it) => {
            return { ...it, filePath: "inline" };
        });
    }
    else {
        return value.results;
    }
}

const options$1 = {
    showLink: false,
    showSummary: false,
    showSelector: true,
};
function toMatchCodeframe(actual, ...rest) {
    var _a;
    /* istanbul ignore next: cant figure out when this would be unset */
    const filename = (_a = this.testPath) !== null && _a !== void 0 ? _a : "inline";
    const results = getResults(filename, actual);
    const enabled = kleur__default.default.enabled;
    kleur__default.default.enabled = false;
    const snapshot = core.codeframe(results, options$1).replace(/\s+$/gm, "");
    kleur__default.default.enabled = enabled;
    /* eslint-disable-next-line @typescript-eslint/no-unsafe-return, @typescript-eslint/no-unsafe-call --
     * the implementation works but the declarations doesn't allow it */
    return jestSnapshot.toMatchSnapshot.call(this, snapshot, ...rest);
}
var toMatchCodeframe$1 = diverge(toMatchCodeframe);

const options = {
    showLink: false,
    showSummary: false,
    showSelector: true,
};
function toMatchInlineCodeframeImpl(context, actual, ...rest) {
    var _a;
    /* istanbul ignore next: cant figure out when this would be unset */
    const filename = (_a = context.testPath) !== null && _a !== void 0 ? _a : "inline";
    const results = getResults(filename, actual);
    const enabled = kleur__default.default.enabled;
    kleur__default.default.enabled = false;
    const snapshot = core.codeframe(results, options).replace(/\s+$/gm, "");
    kleur__default.default.enabled = enabled;
    /* eslint-disable-next-line @typescript-eslint/no-unsafe-return, @typescript-eslint/no-unsafe-call --
     * the implementation works but the declarations doesn't allow it */
    return jestSnapshot.toMatchInlineSnapshot.call(context, snapshot, ...rest);
}
function toMatchInlineCodeframe(actual, ...rest) {
    const context = {
        ...this,
        /* Capture the original stack frames as they are needed by "jest-snapshot"
         * to determine where to write the inline snapshots. When resolving the
         * promise the original stack frames are lost and the snapshot will be
         * written in this files instaed. */
        error: new Error(),
    };
    if (isThenable(actual)) {
        return actual.then((resolved) => toMatchInlineCodeframeImpl(context, resolved, ...rest));
    }
    else {
        return toMatchInlineCodeframeImpl(context, actual, ...rest);
    }
}

exports.toBeInvalid = toBeInvalid$1;
exports.toBeValid = toBeValid$1;
exports.toHTMLValidate = toHTMLValidate$1;
exports.toHaveError = toHaveError$1;
exports.toHaveErrors = toHaveErrors$1;
exports.toMatchCodeframe = toMatchCodeframe$1;
exports.toMatchInlineCodeframe = toMatchInlineCodeframe;
//# sourceMappingURL=jest-lib.js.map
